<?php

namespace QUI\ERP\Products\Affiliate\Fields;

use QUI;

/**
 * Class AffiliateRatingType
 *
 * Field type to configure an affiliate rating (i.e. "3 out of 5 stars" / "60%" etc.)
 */
class AffiliateRatingType extends QUI\ERP\Products\Field\Field
{
    const RATING_TYPE_PERCENTAGE = 1;
    const RATING_TYPE_POINTS     = 2;
    const RATING_TYPE_BOOLEAN    = 3;
    const RATING_TYPE_YES_NO     = 4;
    const RATING_TYPE_TEXT       = 5;

    /**
     * @var bool
     */
    protected $searchable = false;

    /**
     * @var null
     */
    protected $defaultValue = null;
    protected $public       = true;

    /**
     * ProductAttributeList constructor.
     *
     * @param int $fieldId
     * @param array $params
     */
    public function __construct($fieldId, array $params)
    {
        $this->setOptions([
            'criteria'             => [],
            'totalRatingType'      => self::RATING_TYPE_PERCENTAGE,
            'totalRatingMaxPoints' => 0
        ]);

        parent::__construct($fieldId, $params);
    }

    /**
     * Set a field option
     *
     * @param string $option - option name
     * @param mixed $value - option value
     */
    public function setOption($option, $value)
    {
        parent::setOption($option, $value);

        if ($option == 'entries') {
            if (\is_array($value)) {
                foreach ($value as $key => $val) {
                    if (isset($val['selected']) && $val['selected']) {
                        $this->value = $key;
//                        $this->defaultValue = $key;
                    }
                }
            }
        }
    }

    /**
     * @return string
     */
    public function getValue()
    {
        if (!\is_null($this->value)) {
            return $this->value;
        }

        return $this->getDefaultValue();
    }

    /**
     * Return the custom value entry from the user
     *
     * @return string|false
     */
    public function getUserInput()
    {
        if (!\is_null($this->value)) {
            $value = \json_decode($this->value, true);

            if (isset($value[1])) {
                return $value[1];
            }
        }

        return false;
    }

//    /**
//     * Return the FrontendView
//     *
//     * @return UnitSelectFrontendView
//     */
//    public function getFrontendView()
//    {
//        return new UnitSelectFrontendView(
//            $this->getFieldDataForView()
//        );
//    }

    /**
     * @return string
     */
    public function getJavaScriptControl()
    {
        return 'package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType';
    }

    /**
     * @return string
     */
    public function getJavaScriptSettings()
    {
        return 'package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings';
    }

    /**
     * Check the value
     * is the value valid for the field type?
     *
     * @param array
     * @throws \QUI\ERP\Products\Field\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return;
        }

        $invalidExceptionText = [
            'quiqqer/products',
            'exception.field.invalid',
            [
                'fieldId'    => $this->getId(),
                'fieldTitle' => $this->getTitle(),
                'fieldType'  => $this->getType()
            ]
        ];

        if (!\is_string($value) && !\is_array($value)) {
            throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
        }

        if (\is_string($value)) {
            $value = \json_decode($value, true);

            if (\json_last_error() !== \JSON_ERROR_NONE) {
                throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
            }
        }

        $requiredKeys = [
            'totalRating',
            'criteria'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!\array_key_exists($requiredKey, $value)) {
                throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
            }
        }

        if (!empty($value['totalRating']) && !\is_numeric($value['totalRating'])) {
            throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
        }

        if (!\is_array($value['criteria'])) {
            throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
        }

        $requiredCriterionKeys = [
            'rating',
            'ratingComment',
        ];

        $options  = $this->getOptions();
        $criteria = $options['criteria'];

        foreach ($value['criteria'] as $id => $criterion) {
            if (!isset($criteria[$id])) {
                throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
            }

            foreach ($requiredCriterionKeys as $requiredCriterionKey) {
                if (!\array_key_exists($requiredCriterionKey, $criterion)) {
                    throw new QUI\ERP\Products\Field\Exception($invalidExceptionText);
                }
            }
        }
    }

    /**
     * Cleanup the value, so the value is valid
     *
     * @param string|array $value
     * @return array|null
     */
    public function cleanup($value)
    {
        $defaultValue = $this->getDefaultValue();

        if (empty($value)) {
            return $defaultValue;
        }

        if (!\is_string($value) && !\is_array($value)) {
            return $defaultValue;
        }

        if (\is_string($value)) {
            $value = \json_decode($value, true);

            if (\json_last_error() !== \JSON_ERROR_NONE) {
                return $defaultValue;
            }
        }

        $value['totalRating'] = (float)$value['totalRating'];

        foreach ($value['criteria'] as $k => $criterion) {
            if (!empty($criterion['ratingComment']) && !\is_string($criterion['ratingComment'])) {
                $criterion['ratingComment'] = '';
            }

            $value['criteria'][$k] = $criterion;
        }

        return $value;
    }

    /**
     * Get field value title by value
     *
     * @param int $value
     * @param QUI\Locale|null $Locale (optional) - default: QUI::getLocale()
     * @return string
     */
    public function getTitleByValue($value, QUI\Locale $Locale = null)
    {
        return '-';

//        if (empty($value)) {
//            return '-';
//        }
//
//        if ($Locale === null) {
//            $Locale = QUI::getLocale();
//        }
//
//        $options = $this->getOptions();
//        $entries = $options['entries'];
//        $lang    = $Locale->getCurrent();
//
//        if (empty($value['id']) || empty($entries[$value['id']])) {
//            return '-';
//        }
//
//        if (empty($entries[$value['id']]['title'][$lang])) {
//            return '-';
//        }
//
//        return $entries[$value['id']]['title'][$lang];
    }

    /**
     * Return the default value
     *
     * @return mixed|null
     */
    public function getDefaultValue()
    {
//        $defaultValue = null;
//        $options      = $this->getOptions();
//        $entries      = $options['entries'];
//
//        foreach ($entries as $id => $entry) {
//            if ($entry['default']) {
//                if (!empty($entry['quantityInput'])) {
//                    if (!empty($entry['defaultQuantity'])) {
//                        return [
//                            'id'       => $id,
//                            'quantity' => $entry['defaultQuantity']
//                        ];
//                    }
//                } else {
//                    return [
//                        'id'       => $id,
//                        'quantity' => null
//                    ];
//                }
//            }
//        }
//
////        if (isset($entries['piece']) && !empty($entries['piece']['defaultQuantity'])) {
////            return [
////                'id'       => 'piece',
////                'quantity' => $entries['piece']['defaultQuantity']
////            ];
////        }
////
////        if (!empty($entries)) {
////            \reset($entries);
////
////            $id    = \key($entries);
////            $entry = $entries[$id];
////
////            if (!empty($entry['defaultQuantity'])) {
////                return [
////                    'id'       => $id,
////                    'quantity' => $entry['defaultQuantity']
////                ];
////            }
////        }

        return null;
    }

    /**
     * Return the view for the frontend
     *
     * @return AffiliateRatingTypeFrontendView
     */
    public function getFrontendView()
    {
        $Field = new AffiliateRatingTypeFrontendView($this->getFieldDataForView());
        $Field->setProduct($this->Product);

        return $Field;
    }
}
